"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IpfsClusterClient = void 0;
/* eslint-env browser */
const axios_1 = __importDefault(require("axios"));
const form_data_1 = __importDefault(require("form-data"));
const url_1 = require("url");
const request_function_1 = require("./request.function");
const utils_1 = require("./utils");
class IpfsClusterClient {
    constructor(host, username, password) {
        /**
         * base64 encoded
         */
        this.authorizationHeader = '';
        this.clusterHost = new url_1.URL(host);
        if (username && password) {
            this.authorizationHeader = `Basic ${Buffer.from(`${username}:${password}`, 'utf8').toString('base64')}`;
        }
    }
    /**
     * @param {API.RequestOptions} [options]
     */
    version(options) {
        return utils_1.Utils.version(this, this.constructHeaders(), options);
    }
    /**
     * @param {API.RequestOptions} [options]
     * @returns {Promise<API.ClusterInfo>}
     */
    info(options) {
        return utils_1.Utils.info(this, this.constructHeaders(), options);
    }
    constructHeaders(headers = {}) {
        if (this.authorizationHeader) {
            return { ...headers, Authorization: this.authorizationHeader };
        }
        else {
            return headers;
        }
    }
    async addFromFormData(formData, options) {
        const params = utils_1.Utils.encodeAddParams(options);
        try {
            const config = {
                params,
                headers: this.constructHeaders(formData.getHeaders()),
                baseURL: this.clusterHost.href,
                signal: options === null || options === void 0 ? void 0 : options.signal,
            };
            const result = await axios_1.default.post(`add`, formData, config);
            if (!Array.isArray(result.data)) {
                result.data = [result.data];
            }
            const item = result.data[0];
            return {
                name: item.name,
                cid: item.cid['/'],
                size: item.size,
            };
        }
        catch (err) {
            utils_1.Utils.logAxiosError(err);
            throw err;
        }
    }
    /**
     * For endpoint https://docs.ipfs.io/reference/http/api/#api-v0-add
     */
    async addFile(file, options) {
        const formData = new form_data_1.default();
        formData.append('file', file.contents, file.name);
        return await this.addFromFormData(formData, options);
    }
    async addData(file, options) {
        const body = new form_data_1.default();
        body.append('file', file);
        const params = utils_1.Utils.encodeAddParams(options);
        const result = await (0, request_function_1.request)(this.clusterHost.href, 'add', this.constructHeaders(), {
            params,
            method: 'POST',
            body,
            signal: options.signal,
        });
        const data = params.get('stream-channels') ? result : result[0];
        return { ...data, cid: data.cid['/'] };
    }
    async addDirectory(files, options) {
        const formData = new form_data_1.default();
        for (const f of files) {
            formData.append('file', f.contents, f.name);
        }
        const results = await (0, request_function_1.request)(this.clusterHost.href, 'add', this.constructHeaders(formData.getHeaders()), {
            params: {
                ...utils_1.Utils.encodeAddParams(options),
                'stream-channels': false,
                'wrap-with-directory': true,
            },
            method: 'POST',
            body: formData,
            signal: options === null || options === void 0 ? void 0 : options.signal,
        });
        for (const f of results) {
            f.cid = f.cid['/'];
        }
        return results;
    }
    async pin(cid, options) {
        const path = cid.startsWith('/') ? `pins${cid}` : `pins/${cid}`;
        const data = await (0, request_function_1.request)(this.clusterHost.href, path, this.constructHeaders(), {
            params: utils_1.Utils.getPinParams(options),
            method: 'POST',
            signal: options.signal,
        });
        return utils_1.Utils.toPinResponse(data);
    }
    async unpin(cid, options) {
        const path = cid.startsWith('/') ? `pins${cid}` : `pins/${cid}`;
        const data = await (0, request_function_1.request)(this.clusterHost.href, path, this.constructHeaders(), {
            ...options,
            method: 'DELETE',
        });
        return utils_1.Utils.toPinResponse(data);
    }
    async pinls(options) {
        const path = `allocations`;
        const data = await (0, request_function_1.request)(this.clusterHost.href, path, this.constructHeaders(), options);
        return utils_1.Utils.toPinResponse(data);
    }
    async status(cid, options) {
        const path = `pins/${encodeURIComponent(cid)}`;
        const data = await (0, request_function_1.request)(this.clusterHost.href, path, this.constructHeaders(), {
            params: { local: options === null || options === void 0 ? void 0 : options.local },
            signal: options === null || options === void 0 ? void 0 : options.signal,
        });
        const peer_map = data.peer_map;
        let peerMap = {};
        if (peer_map) {
            peerMap = Object.fromEntries(Object.entries(peer_map).map(([k, v]) => [
                k,
                {
                    peerName: v.peername,
                    status: v.status,
                    timestamp: new Date(v.timestamp),
                    error: v.error,
                },
            ]));
        }
        return { cid: data.cid['/'], name: data.name, peerMap };
    }
    async allocation(cid, options) {
        const path = `allocations/${encodeURIComponent(cid)}`;
        const data = await (0, request_function_1.request)(this.clusterHost.href, path, this.constructHeaders(), options);
        return utils_1.Utils.toPinResponse(data);
    }
    async recover(cid, options) {
        const path = `pins/${encodeURIComponent(cid)}/recover`;
        const data = await (0, request_function_1.request)(this.clusterHost.href, path, this.constructHeaders(), {
            method: 'POST',
            params: { local: options === null || options === void 0 ? void 0 : options.local },
            signal: options === null || options === void 0 ? void 0 : options.signal,
        });
        const peer_map = data.peer_map;
        let peerMap = {};
        if (peer_map) {
            peerMap = Object.fromEntries(Object.entries(peer_map).map(([k, v]) => [
                k,
                {
                    peerName: v.peername,
                    status: v.status,
                    timestamp: new Date(v.timestamp),
                    error: v.error,
                },
            ]));
        }
        return { cid: data.cid['/'], name: data.name, peerMap };
    }
    /**
     * @param {API.RequestOptions} [options]
     * @returns {Promise<string[]>}
     */
    metricNames(options) {
        return (0, request_function_1.request)(this.clusterHost.href, 'monitor/metrics', this.constructHeaders(), options);
    }
}
exports.IpfsClusterClient = IpfsClusterClient;
