"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Utils = void 0;
const url_1 = require("url");
const request_function_1 = require("./request.function");
const axios_1 = __importDefault(require("axios"));
class Utils {
    /**
     * Gets cluster version
     *
     * @param {API.Config} cluster
     * @param {API.RequestOptions} [options]
     * @returns {Promise<string>}
     */
    static async version(cluster, headers, options) {
        const result = await (0, request_function_1.request)(cluster.clusterHost.href, 'version', headers, options);
        if (typeof result.version !== 'string') {
            throw new Error(`failed to parse version from response the body: ${JSON.stringify(result, null, 2)}`);
        }
        return result.version;
    }
    /**
     * Gets cluster information
     *
     * @param {API.Config} cluster
     * @param {API.RequestOptions} [options]
     * @returns {Promise<API.ClusterInfo>}
     */
    static async info(cluster, headers, options) {
        var _a;
        const result = await (0, request_function_1.request)(cluster.clusterHost.href, 'id', headers, options);
        const failure = result.error || ((_a = result.ipfs) === null || _a === void 0 ? void 0 : _a.error) || '';
        if (failure.length > 0) {
            throw new Error(`cluster id response has failure: ${JSON.stringify(result, null, 2)}`);
        }
        const { id, addresses, version, commit, peername: peerName, rpc_protocol_version: rpcProtocolVersion, cluster_peers: clusterPeers, cluster_peers_addresses: clusterPeersAddresses, ipfs, } = result;
        return {
            id,
            addresses,
            version,
            commit,
            peerName,
            rpcProtocolVersion,
            clusterPeers,
            clusterPeersAddresses,
            ipfs,
        };
    }
    static encodeAddParams(options = {}) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
        if (!options)
            return new url_1.URLSearchParams();
        const kvPairs = this.filterUndefinedProperties({
            ...this.getPinParams(options),
            local: (_a = options.local) === null || _a === void 0 ? void 0 : _a.toString(),
            recursive: (_b = options === null || options === void 0 ? void 0 : options.recursive) === null || _b === void 0 ? void 0 : _b.toString(),
            hidden: (_c = options.hidden) === null || _c === void 0 ? void 0 : _c.toString(),
            wrap: (_d = options.wrap) === null || _d === void 0 ? void 0 : _d.toString(),
            shard: (_e = options.shard) === null || _e === void 0 ? void 0 : _e.toString(),
            // stream-channels=false means buffer entire response in cluster before returning.
            // MAY avoid weird edge-cases with prematurely closed streams
            // see: https://github.com/web3-storage/web3.storage/issues/323
            'stream-channels': (_g = (_f = options.streamChannels) === null || _f === void 0 ? void 0 : _f.toString()) !== null && _g !== void 0 ? _g : 'false',
            format: options.format,
            layout: options.layout,
            chunker: options.chunker,
            'raw-leaves': (_j = (_h = options.rawLeaves) === null || _h === void 0 ? void 0 : _h.toString()) !== null && _j !== void 0 ? _j : 'true',
            progress: (_k = options.progress) === null || _k === void 0 ? void 0 : _k.toString(),
            'cid-version': (_m = (_l = options.cidVersion) === null || _l === void 0 ? void 0 : _l.toString()) !== null && _m !== void 0 ? _m : '1',
            hash: options.hashFun,
            'no-copy': (_o = options.noCopy) === null || _o === void 0 ? void 0 : _o.toString(),
        });
        return new url_1.URLSearchParams(kvPairs);
    }
    static getPinParams(options) {
        var _a, _b, _c, _d, _e, _f;
        return {
            name: options.name,
            mode: options.mode,
            'replication-min': (_a = options.replicationFactorMin) === null || _a === void 0 ? void 0 : _a.toString(),
            'replication-max': (_b = options.replicationFactorMax) === null || _b === void 0 ? void 0 : _b.toString(),
            'shard-size': (_c = options.shardSize) === null || _c === void 0 ? void 0 : _c.toString(),
            'user-allocations': (_d = options.userAllocations) === null || _d === void 0 ? void 0 : _d.join(','),
            'expire-at': (_e = options.expireAt) === null || _e === void 0 ? void 0 : _e.toISOString(),
            'pin-update': options.pinUpdate,
            origins: (_f = options.origins) === null || _f === void 0 ? void 0 : _f.join(','),
            ...this.encodeMetadata(options.metadata || {}),
        };
    }
    static filterUndefinedProperties(obj) {
        return Object.fromEntries(Object.entries(obj).filter(([, v]) => v != null));
    }
    static encodeMetadata(metadata) {
        return Object.fromEntries(Object.entries(metadata).map(([k, v]) => [`meta-${k}`, v]));
    }
    static toPinResponse(data) {
        return {
            replicationFactorMin: data.replication_factor_min,
            replicationFactorMax: data.replication_factor_max,
            name: data.name,
            mode: data.mode,
            shardSize: data.shard_size,
            userAllocations: data.user_allocations,
            expireAt: new Date(data.expire_at),
            metadata: data.metadata,
            pinUpdate: data.pin_update,
            cid: data.cid['/'],
            type: data.type,
            allocations: data.allocations,
            maxDepth: data.max_depth,
            reference: data.reference,
        };
    }
    static logAxiosError(err) {
        var _a, _b;
        if (axios_1.default.isAxiosError(err)) {
            console.error(`response ${err.message}`);
            console.error(`status ${err.code}`);
            console.error((_a = err.response) === null || _a === void 0 ? void 0 : _a.data);
            console.error((_b = err.response) === null || _b === void 0 ? void 0 : _b.statusText);
        }
    }
}
exports.Utils = Utils;
